[freeside-commits] freeside/FS/FS payinfo_Mixin.pm, 1.1, 1.2 cust_pay_batch.pm, 1.17, 1.18 payby.pm, 1.9, 1.10

Ivan,,, ivan at wavetail.420.am
Tue Dec 19 02:02:08 PST 2006


Update of /home/cvs/cvsroot/freeside/FS/FS
In directory wavetail:/tmp/cvs-serv18716/FS/FS

Modified Files:
	payinfo_Mixin.pm cust_pay_batch.pm payby.pm 
Log Message:
clean up payinfo_Mixin to use payby.pm for payby info and have card masking full 6-digit BIN prefix for card identification.  have cust_pay_batch use payinfo_Mixin.  require B:CC 0.30 for mask-aware cardtype().  fix payment reports to use mask too if available, so credit card type selection still works with encryption.

Index: cust_pay_batch.pm
===================================================================
RCS file: /home/cvs/cvsroot/freeside/FS/FS/cust_pay_batch.pm,v
retrieving revision 1.17
retrieving revision 1.18
diff -u -d -r1.17 -r1.18
--- cust_pay_batch.pm	1 Dec 2006 06:31:55 -0000	1.17
+++ cust_pay_batch.pm	19 Dec 2006 10:02:06 -0000	1.18
@@ -3,10 +3,11 @@
 use strict;
 use vars qw( @ISA $DEBUG );
 use FS::Record qw(dbh qsearch qsearchs);
+use FS::payinfo_Mixin;
 use FS::part_bill_event qw(due_events);
 use Business::CreditCard 0.28;
 
- at ISA = qw( FS::Record );
+ at ISA = qw( FS::Record FS::payinfo_Mixin );
 
 # 1 is mostly method/subroutine entry and options
 # 2 traces progress of some operations
@@ -124,7 +125,7 @@
 
   my $error = 
       $self->ut_numbern('paybatchnum')
-    || $self->ut_numbern('trancode') #depriciated
+    || $self->ut_numbern('trancode') #deprecated
     || $self->ut_money('amount')
     || $self->ut_number('invnum')
     || $self->ut_number('custnum')
@@ -142,11 +143,7 @@
   $self->first =~ /^([\w \,\.\-\']+)$/ or return "Illegal first name";
   $self->first($1);
 
-  $self->payby =~ /^(CARD|CHEK|LECB|BILL|COMP|PREP|CASH|WEST|MCRD)$/
-    or return "Illegal payby";
-  $self->payby($1);
-
-  $error = FS::payby::payinfo_check($self->payby, \$self->payinfo);
+  $error = $self->payinfo_check();
   return $error if $error;
 
   if ( $self->exp eq '' ) {

Index: payinfo_Mixin.pm
===================================================================
RCS file: /home/cvs/cvsroot/freeside/FS/FS/payinfo_Mixin.pm,v
retrieving revision 1.1
retrieving revision 1.2
diff -u -d -r1.1 -r1.2
--- payinfo_Mixin.pm	14 Dec 2006 06:00:45 -0000	1.1
+++ payinfo_Mixin.pm	19 Dec 2006 10:02:06 -0000	1.2
@@ -2,6 +2,7 @@
 
 use strict;
 use Business::CreditCard;
+use FS::payby;
 
 =head1 NAME
 
@@ -13,7 +14,7 @@
 use vars qw(@ISA);
 @ISA = qw( FS::payinfo_Mixin FS::Record );
 
-=head1 DESCRIPTION
+=had1 DESCRIPTION
 
 This is a mixin class for records that contain payinfo. 
 
@@ -25,7 +26,7 @@
 Bad Card Stuff - In the Future (Integrate Banned Pay)
 Currency - In the Future
 
-=head1 fields
+=head1 FIELDS
 
 =over 4
 
@@ -37,12 +38,12 @@
 'CARD' (credit card - automatic), 'DCRD' (credit card - on-demand),
 'CHEK' (electronic check - automatic), 'DCHK' (electronic check - on-demand),
 'LECB' (Phone bill billing), 'BILL' (billing), 'COMP' (free), or
-'PREPAY' (special billing type: applies a credit - see L<FS::prepay_credit> and sets billing type to I<BILL>)
+'PREPAY' (special billing type: applies a credit and sets billing type to I<BILL> - see L<FS::prepay_credit>)
 
 For Refunds (cust_refund):
 'CARD' (credit cards), 'CHEK' (electronic check/ACH),
 'LECB' (Phone bill billing), 'BILL' (billing), 'CASH' (cash),
-'WEST' (Western Union), 'MCRD' (Manual credit card), 'CBAK' Chargeback, or 'COMP' (free),
+'WEST' (Western Union), 'MCRD' (Manual credit card), 'CBAK' Chargeback, or 'COMP' (free)
 
 
 For Payments (cust_pay):
@@ -52,15 +53,16 @@
 'COMP' (free) is depricated as a payment type in cust_pay
 
 =cut 
- 
-sub payby {
-  my($self,$payby) = @_;
-  if ( defined($payby) ) {
-    $self->setfield('payby', $payby);
-  } 
-  return $self->getfield('payby')
-}
 
+# was this supposed to do something?
+ 
+#sub payby {
+#  my($self,$payby) = @_;
+#  if ( defined($payby) ) {
+#    $self->setfield('payby', $payby);
+#  } 
+#  return $self->getfield('payby')
+#}
 
 =item payinfo
 
@@ -99,6 +101,7 @@
     }
   } else {
 #    warn "This doesn't work for other tables besides cust_main
+    '';
   } 
 }
 
@@ -125,7 +128,13 @@
   return $paymask;
 }
 
-=item mask_payinfo()
+=back
+
+=head1 METHODS
+
+=over 4
+
+=item mask_payinfo
 
 This method converts the payment info (credit card, bank account, etc.) into a masked string.
 
@@ -142,7 +151,7 @@
   } else {
     # if not, mask it...
     if ($payby eq 'CARD' || $payby eq 'DCRD' || $payby eq 'MCRD') { # Credit Cards (Show first and last four)
-      $paymask = substr($payinfo,0,4). 'x'x(length($payinfo)-8). substr($payinfo,(length($payinfo)-4));
+      $paymask = substr($payinfo,0,6). 'x'x(length($payinfo)-10). substr($payinfo,(length($payinfo)-4));
     } elsif ($payby eq 'CHEK' ||
              $payby eq 'DCHK' ) { # Checks (Show last 2 @ bank)
       my( $account, $aba ) = split('@', $payinfo );
@@ -154,15 +163,10 @@
   return $paymask;
 }
 
-=back
-
-
-=head1 METHODS
-
-=over 4
-
 =item payinfo_check
 
+Checks payby and payinfo.
+
 For Customers (cust_main):
 'CARD' (credit card - automatic), 'DCRD' (credit card - on-demand),
 'CHEK' (electronic check - automatic), 'DCHK' (electronic check - on-demand),
@@ -182,33 +186,11 @@
 
 =cut
 
-
-
-
-
 sub payinfo_check {
   my $self = shift;
 
-  # Make sure it's a valid payby
-  $self->payby =~ /^(CARD|DCRD|CHEK|DCHK|LECB|BILL|COMP|PREPAY|CASH|WEST|MCRD|PREP|CBAK)$/
-    or return "Illegal payby (overall payinfo_check)";
-  $self->payby($1);
-
-
-  # Okay some aren't valid depending on table
-  if ($self->table eq 'cust_main') {
-    if ($self->payby =~ /^(CASH|WEST|MCRD|PREP|CBAK)$/) {
-      return "Illegal payby (cust_main)";
-    }
-  } elsif ($self->table eq 'cust_refund') {
-    if ($self->payby =~ /^(DCRD|DCHK|PREPAY|PREP)$/) {
-      return "Illegal payby (cust_refund)";
-    }
-  } elsif ($self->table eq 'cust_pay') {
-    if ($self->payby =~ /^(DCRD|DCHK|PREPAY|CBAK)$/) {
-      return "Illegal payby (cust_pay)";
-    }
-  }
+  FS::payby->can_payby($self->table, $self->payby)
+    or return "Illegal payby";
 
   if ( $self->payby eq 'CARD' ) {
     my $payinfo = $self->payinfo;
@@ -218,8 +200,8 @@
       $self->payinfo =~ /^(\d{13,16})$/
         or return "Illegal (mistyped?) credit card number (payinfo)";
       $self->payinfo($1);
-      Business::CreditCard::validate($self->payinfo) or return "Illegal credit card number";
-      return "Unknown card type" if Business::CreditCard::cardtype($self->payinfo) eq "Unknown";
+      validate($self->payinfo) or return "Illegal credit card number";
+      return "Unknown card type" if cardtype($self->payinfo) eq "Unknown";
     } else {
       $self->payinfo('N/A');
     }
@@ -229,15 +211,13 @@
   }
 }
 
-
-
 =head1 BUGS
 
 Have to add the future items...
 
 =head1 SEE ALSO
 
-L<FS::Record>
+L<FS::payby>, L<FS::Record>
 
 =cut
 



More information about the freeside-commits mailing list
Index: payby.pm
===================================================================
RCS file: /home/cvs/cvsroot/freeside/FS/FS/payby.pm,v
retrieving revision 1.9
retrieving revision 1.10
diff -u -d -r1.9 -r1.10
--- payby.pm	8 Oct 2006 08:17:06 -0000	1.9
+++ payby.pm	19 Dec 2006 10:02:06 -0000	1.10
@@ -18,6 +18,8 @@
 
   my @payby = FS::payby->payby;
 
+  my $bool = FS::payby->can_payby('cust_main', 'CARD');
+
   tie my %payby, 'Tie::IxHash', FS::payby->payby2longname
 
   my @cust_payby = FS::payby->cust_payby;
@@ -38,7 +40,7 @@
 
 # paybys can be any/all of:
 # - a customer payment type (cust_main.payby)
 # - a payment or refund type (cust_pay.payby)
 # - an event type (part_bill_event.payby)
 
 tie %hash, 'Tie::IxHash',
@@ -96,6 +98,13 @@
     tinyname  => 'comp',
     shortname => 'Complimentary',
     longname  => 'Complimentary',
+    cust_pay  => '', # (free) is depricated as a payment type in cust_pay
+  },
+  'CBAK' => {
+    tinyname  => 'chargeback',
+    shortname => 'Chargeback',
+    longname  => 'Chargeback',
+    cust_main => '', # not a customer type
   },
   'DCLN' => {  # This is only an event.
     tinyname  => 'declined',
@@ -103,7 +112,7 @@
     longname  => 'Batch declined payment',
 
     #its neither of these..
-    #cust_main => '',
+    cust_main => '',
     cust_pay  => '',
 
   },
@@ -113,6 +122,18 @@
   keys %hash;
 }
 
+sub can_payby {
+  my( $self, $table, $payby ) = @_;
+
+  #return "Illegal payby" unless $hash{$payby};
+  return 0 unless $hash{$payby};
+
+  $table = 'cust_pay' if $table eq 'cust_pay_batch' || $table eq 'cust_refund';
+  return 0 if exists( $hash{$payby}->{$table} );
+
+  return 1;
+}
+
 sub payby2longname {
   my $self = shift;
   map { $_ => $hash{$_}->{longname} } $self->payby;
@@ -153,28 +174,6 @@
   map { $_ => $hash{$_}->{longname} } $self->cust_payby;
 }
 
-sub payinfo_check{
-  my($payby, $payinforef) = @_;
-
-  if ($payby eq 'CARD') {
-    $$payinforef =~ s/\D//g;
-    if ($$payinforef){
-      $$payinforef =~ /^(\d{13,16})$/
-        or return "Illegal (mistyped?) credit card number (payinfo)";
-      $$payinforef = $1;
-      validate($$payinforef) or return "Illegal credit card number";
-      return "Unknown card type" if cardtype($$payinforef) eq "Unknown";
-    } else {
-      $$payinforef="N/A";
-    }
-  } else {
-    $$payinforef =~ /^([\w \!\@\#\$\%\&\(\)\-\+\;\:\'\"\,\.\?\/\=]*)$/
-    or return "Illegal text (payinfo)";
-    $$payinforef = $1;
-  }
-  '';
-}
-
 =back
 
 =head1 BUGS
@@ -38,7 +40,7 @@
 # paybys can be any/all of:
 # - a customer payment type (cust_main.payby)
-# - a payment or refund type (cust_pay.payby)
+# - a payment or refund type (cust_pay.payby, cust_pay_batch.payby, cust_refund.payby)
 # - an event type (part_bill_event.payby)
 
